@@ -4,7 +4,7 @@
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# $Id: ImpactPacket.py 737 2013-04-09 14:11:32Z bethus@gmail.com $
+# $Id: ImpactPacket.py 946 2013-11-26 17:50:36Z aurelianocalvo $
 #
 # Description:
 #  Network packet codecs basic building blocks.
@@ -43,7 +43,7 @@ class PacketBuffer:
     packet's raw buffer. All the packet classes derive from this one.
 
     The byte, word, long and ip_address getters and setters accept
-    negative indeces, having these the a similar effect as in a
+    negative indexes, having these the a similar effect as in a
     regular Python sequence slice.
     """
 
@@ -185,7 +185,7 @@ class PacketBuffer:
     def __validate_index(self, index, size):
         """This method performs two tasks: to allocate enough space to
         fit the elements at positions index through index+size, and to
-        adjust negative indeces to their absolute equivalent.
+        adjust negative indexes to their absolute equivalent.
         """
 
         orig_index = index
@@ -383,7 +383,7 @@ class Header(PacketBuffer,ProtocolLayer):
     def get_packet(self):
         """Returns the raw representation of this packet and its
         children as a string. The output from this method is a packet
-        ready to be transmited over the wire.
+        ready to be transmitted over the wire.
         """
         self.calculate_checksum()
 
@@ -421,7 +421,7 @@ class Header(PacketBuffer,ProtocolLayer):
 
     def get_header_size(self):
         "Return the size of this header, that is, not counting neither the size of the children nor of the parents."
-        raise RuntimeError("Method %s.get_header_size must be overriden." % self.__class__)
+        raise RuntimeError("Method %s.get_header_size must be overridden." % self.__class__)
 
     def list_as_hex(self, aList):
         if len(aList):
@@ -456,7 +456,7 @@ class Header(PacketBuffer,ProtocolLayer):
         ltmp = self.list_as_hex(self.get_bytes().tolist())
 
         if self.child():
-            ltmp.append(['\n', self.child().__str__()])
+            ltmp.append(['\n', str(self.child())])
 
         if len(ltmp)>0:
             return string.join(ltmp, '')
@@ -482,7 +482,7 @@ class Data(Header):
     def __init__(self, aBuffer = None):
         Header.__init__(self)
         if aBuffer:
-           self.set_data(aBuffer)
+            self.set_data(aBuffer)
 
     def set_data(self, data):
         self.set_bytes_from_string(data)
@@ -491,33 +491,145 @@ class Data(Header):
         return len(self.get_bytes())
 
 
+class EthernetTag(PacketBuffer):
+    """Represents a VLAN header specified in IEEE 802.1Q and 802.1ad.
+       Provides methods for convenient manipulation with header fields."""
+
+    def __init__(self, value=0x81000000):
+        PacketBuffer.__init__(self, 4)
+        self.set_long(0, value)
+
+    def get_tpid(self):
+        """Returns Tag Protocol Identifier"""
+        return self.get_word(0)
+
+    def set_tpid(self, value):
+        """Sets Tag Protocol Identifier"""
+        return self.set_word(0, value)
+
+    def get_pcp(self):
+        """Returns Priority Code Point"""
+        return (self.get_byte(2) & 0xE0) >> 5
+
+    def set_pcp(self, value):
+        """Sets Priority Code Point"""
+        orig_value = self.get_byte(2)
+        self.set_byte(2, (orig_value & 0x1F) | ((value & 0x07) << 5))
+
+    def get_dei(self):
+        """Returns Drop Eligible Indicator"""
+        return (self.get_byte(2) & 0x10) >> 4
+
+    def set_dei(self, value):
+        """Sets Drop Eligible Indicator"""
+        orig_value = self.get_byte(2)
+        self.set_byte(2, orig_value | 0x10 if value else orig_value & 0xEF)
+
+    def get_vid(self):
+        """Returns VLAN Identifier"""
+        return self.get_word(2) & 0x0FFF
+
+    def set_vid(self, value):
+        """Sets VLAN Identifier"""
+        orig_value = self.get_word(2)
+        self.set_word(2, (orig_value & 0xF000) | (value & 0x0FFF))
+
+    def __str__(self):
+        priorities = (
+            'Best Effort',
+            'Background',
+            'Excellent Effort',
+            'Critical Applications',
+            'Video, < 100 ms latency and jitter',
+            'Voice, < 10 ms latency and jitter',
+            'Internetwork Control',
+            'Network Control')
+
+        pcp = self.get_pcp()
+        return '\n'.join((
+            '802.1Q header: 0x{0:08X}'.format(self.get_long(0)),
+            'Priority Code Point: {0} ({1})'.format(pcp, priorities[pcp]),
+            'Drop Eligible Indicator: {0}'.format(self.get_dei()),
+            'VLAN Identifier: {0}'.format(self.get_vid())))
+
 
 class Ethernet(Header):
     def __init__(self, aBuffer = None):
         Header.__init__(self, 14)
+        self.tag_cnt = 0
         if(aBuffer):
             self.load_header(aBuffer)
 
     def set_ether_type(self, aValue):
         "Set ethernet data type field to 'aValue'"
-        self.set_word(12, aValue)
+        self.set_word(12 + 4*self.tag_cnt, aValue)
 
     def get_ether_type(self):
         "Return ethernet data type field"
-        return self.get_word(12)
+        return self.get_word(12 + 4*self.tag_cnt)
+
+    def get_tag(self, index):
+        """Returns an EthernetTag initialized from index-th VLAN tag.
+           The tags are numbered from 0 to self.tag_cnt-1 as they appear in the frame.
+           It is possible to use negative indexes as well."""
+        index = self.__validate_tag_index(index)
+        return EthernetTag(self.get_long(12+4*index))
+
+    def set_tag(self, index, tag):
+        """Sets the index-th VLAN tag to contents of an EthernetTag object.
+           The tags are numbered from 0 to self.tag_cnt-1 as they appear in the frame.
+           It is possible to use negative indexes as well."""
+        index = self.__validate_tag_index(index)
+        pos = 12 + 4*index
+        for i,val in enumerate(tag.get_bytes()):
+            self.set_byte(pos+i, val)
+
+    def push_tag(self, tag, index=0):
+        """Inserts contents of an EthernetTag object before the index-th VLAN tag.
+           Index defaults to 0 (the top of the stack)."""
+        if index < 0:
+            index += self.tag_cnt
+        pos = 12 + 4*max(0, min(index, self.tag_cnt))
+        data = self.get_bytes()
+        data[pos:pos] = tag.get_bytes()
+        self.set_bytes(data)
+        self.tag_cnt += 1
+
+    def pop_tag(self, index=0):
+        """Removes the index-th VLAN tag and returns it as an EthernetTag object.
+           Index defaults to 0 (the top of the stack)."""
+        index = self.__validate_tag_index(index)
+        pos = 12 + 4*index
+        tag = self.get_long(pos)
+        data = self.get_bytes()
+        del data[pos:pos+4]
+        self.set_bytes(data)
+        self.tag_cnt -= 1
+        return EthernetTag(tag)
+
+    def load_header(self, aBuffer):
+        self.tag_cnt = 0
+        while aBuffer[12+4*self.tag_cnt:14+4*self.tag_cnt] in ('\x81\x00', '\x88\xa8', '\x91\x00'):
+            self.tag_cnt += 1
+
+        hdr_len = self.get_header_size()
+        diff = hdr_len - len(aBuffer)
+        if diff > 0:
+            aBuffer += '\x00'*diff
+        self.set_bytes_from_string(aBuffer[:hdr_len])
 
     def get_header_size(self):
         "Return size of Ethernet header"
-        return 14
+        return 14 + 4*self.tag_cnt
 
     def get_packet(self):
 
         if self.child():
             try:
-               self.set_ether_type(self.child().ethertype)
+                self.set_ether_type(self.child().ethertype)
             except:
-               " an Ethernet packet may have a Data() "
-               pass
+                " an Ethernet packet may have a Data() "
+                pass
         return Header.get_packet(self)
 
     def get_ether_dhost(self):
@@ -547,9 +659,17 @@ class Ethernet(Header):
         tmp_str = 'Ether: ' + self.as_eth_addr(self.get_ether_shost()) + ' -> '
         tmp_str += self.as_eth_addr(self.get_ether_dhost())
         if self.child():
-            tmp_str += '\n' + self.child().__str__()
+            tmp_str += '\n' + str( self.child())
         return tmp_str
 
+    def __validate_tag_index(self, index):
+        """Adjusts negative indices to their absolute equivalents.
+           Raises IndexError when out of range <0, self.tag_cnt-1>."""
+        if index < 0:
+            index += self.tag_cnt
+        if index < 0 or index >= self.tag_cnt:
+            raise IndexError("Tag index out of range")
+        return index
 
 # Linux "cooked" capture encapsulation.
 # Used, for instance, for packets returned by the "any" interface.
@@ -631,7 +751,7 @@ class LinuxSLL(Header):
         addr = hexlify(self.get_addr()[0:alen])
         ss.append("Linux SLL: addr=%s type=`%s'" % (addr, self.get_type_desc()))
         if self.child():
-            ss.append(self.child().__str__())
+            ss.append(str(self.child()))
 
         return '\n'.join(ss)
 
@@ -695,15 +815,15 @@ class IP(Header):
 
 
 
-  #  def calculate_checksum(self, buffer = None):
-  #      tmp_value = self.get_ip_sum()
-  #      if self.auto_checksum and (not tmp_value):
-  #          if buffer:
-  #              tmp_bytes = buffer
-  #          else:
-  #              tmp_bytes = self.bytes[0:self.get_header_size()]
-  #
-  #          self.set_ip_sum(self.compute_checksum(tmp_bytes))
+    #  def calculate_checksum(self, buffer = None):
+    #      tmp_value = self.get_ip_sum()
+    #      if self.auto_checksum and (not tmp_value):
+    #          if buffer:
+    #              tmp_bytes = buffer
+    #          else:
+    #              tmp_bytes = self.bytes[0:self.get_header_size()]
+    #
+    #          self.set_ip_sum(self.compute_checksum(tmp_bytes))
 
 
     def get_pseudo_header(self):
@@ -977,9 +1097,9 @@ class IP(Header):
         if self.get_ip_rf(): flags += 'RF '
         tmp_str = 'IP%s%s -> %s ' % (flags, self.get_ip_src(),self.get_ip_dst())
         for op in self.__option_list:
-            tmp_str += '\n' + op.__str__()
+            tmp_str += '\n' + str(op)
         if self.child():
-            tmp_str += '\n' + self.child().__str__()
+            tmp_str += '\n' + str(self.child())
         return tmp_str
 
 
@@ -1035,7 +1155,7 @@ class IPOption(PacketBuffer):
             self.set_flags(0)
         else:
             if not size:
-                raise ImpactPacketError, "Size required for this type"
+                raise ImpactPacketException, "Size required for this type"
             PacketBuffer.__init__(self,size)
             self.set_code(opcode)
             self.set_len(size)
@@ -1182,7 +1302,7 @@ class UDP(Header):
     def __str__(self):
         tmp_str = 'UDP %d -> %d' % (self.get_uh_sport(), self.get_uh_dport())
         if self.child():
-            tmp_str += '\n' + self.child().__str__()
+            tmp_str += '\n' + str(self.child())
         return tmp_str
 
     def get_packet(self):
@@ -1347,7 +1467,7 @@ class TCP(Header):
     def reset_FIN(self):
         return self.reset_flags(1)
 
-    # Overriden Methods
+    # Overridden Methods
 
     def get_header_size(self):
         return 20 + len(self.get_padded_options())
@@ -1462,10 +1582,10 @@ class TCP(Header):
             tmp_str += 'urg '
         tmp_str += '%d -> %d' % (self.get_th_sport(), self.get_th_dport())
         for op in self.__option_list:
-            tmp_str += '\n' + op.__str__()
+            tmp_str += '\n' + str(op)
 
         if self.child():
-            tmp_str += '\n' + self.child().__str__()
+            tmp_str += '\n' + str(self.child())
         return tmp_str
 
 
@@ -1796,7 +1916,7 @@ class ICMP(Header):
         tmp_str = 'ICMP type: ' + self.get_type_name(tmp_type)
         tmp_str+= ' code: ' + self.get_code_name(tmp_type, tmp_code)
         if self.child():
-            tmp_str += '\n' + self.child().__str__()
+            tmp_str += '\n' + str( self.child() )
         return tmp_str
 
     def isDestinationUnreachable(self):
@@ -1818,8 +1938,8 @@ class ICMP(Header):
         return self.isDestinationUnreachable() and (self.get_icmp_code() == 2)
 
     def isQuery(self):
-         tmp_dict = {8:'',  9:'',  10:'', 13:'', 14:'', 15:'', 16:'', 17:'', 18:''}
-         return tmp_dict.has_key(self.get_icmp_type())
+        tmp_dict = {8:'',  9:'',  10:'', 13:'', 14:'', 15:'', 16:'', 17:'', 18:''}
+        return tmp_dict.has_key(self.get_icmp_type())
 
 class IGMP(Header):
     protocol = 2
@@ -1857,7 +1977,7 @@ class IGMP(Header):
 
     def get_type_name(self, aType):
         tmp_dict = {0x11:'HOST MEMBERSHIP QUERY ', 0x12:'v1 HOST MEMBERSHIP REPORT ', 0x13:'IGMP DVMRP ', 0x14:' PIM ', 0x16:'v2 HOST MEMBERSHIP REPORT ', 0x17:'HOST LEAVE MESSAGE ', 0x1e:'MTRACE REPLY ', 0X1f:'MTRACE QUERY '}
-        answer = tmp_type.get(aType, 'UNKNOWN TYPE OR VERSION ')
+        answer = tmp_dict.get(aType, 'UNKNOWN TYPE OR VERSION ')
         return answer
 
     def calculate_checksum(self):
@@ -1865,11 +1985,10 @@ class IGMP(Header):
             self.set_igmp_cksum(self.compute_checksum(self.get_bytes()))
 
     def __str__(self):
-        knowcode = 0
         tmp_str = 'IGMP: ' + self.get_type_name(self.get_igmp_type())
-        tmp_str += 'Group: ' +  socket.inet_ntoa(pack('!L',self.get_igmp_group()))
+        tmp_str += 'Group: ' +  socket.inet_ntoa(struct.pack('!L',self.get_igmp_group()))
         if self.child():
-            tmp_str += '\n' + self.child().__str__()
+            tmp_str += '\n' + str(self.child())
         return tmp_str
 
 
@@ -1986,7 +2105,7 @@ class ARP(Header):
         tmp_str += '\n' + self.as_pro(self.get_ar_spa()) + ' -> '
         tmp_str += self.as_pro(self.get_ar_tpa())
         if self.child():
-            tmp_str += '\n' + self.child().__str__()
+            tmp_str += '\n' + str(self.child())
         return tmp_str
 
 def example(): #To execute an example, remove this line
