@@ -4,7 +4,7 @@
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# $Id: nmb.py 653 2012-08-13 18:44:04Z bethus@gmail.com $
+# $Id: nmb.py 763 2013-05-17 19:30:16Z bethus@gmail.com $
 #
 
 
@@ -885,7 +885,7 @@ class NetBIOSTCPSession(NetBIOSSession):
                 ready, _, _ = select.select([self._sock.fileno() ], [ ], [ ], 0)
                 
                 if not ready:
-                    if time_left == 0:
+                    if time_left <= 0:
                         raise NetBIOSTimeout
                     else:
                         time.sleep(CHUNK_TIME)
